<?php

namespace Drupal\decoupled_kit_block\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Url;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\block\BlockRepositoryInterface;

/**
 * Class DefaultController.
 */
class DefaultController extends ControllerBase {

  /**
   * Drupal\Core\DependencyInjection\ContainerInjectionInterface definition.
   *
   * @var \Drupal\Core\DependencyInjection\ContainerInjectionInterface
   */
  protected $decoupledKit;

  /**
   * Drupal\Core\Theme\ThemeManagerInterface definition.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * Drupal\Core\Path\PathMatcherInterface definition.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * Drupal\Core\Render\RendererInterface definition.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->decoupledKit = $container->get('decoupled_kit');
    $instance->themeManager = $container->get('theme.manager');
    $instance->pathMatcher = $container->get('path.matcher');
    $instance->renderer = $container->get('renderer');
    return $instance;
  }

  /**
   * Get blocks data for current page.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   Request object.
   *
   * @return string
   *   Return JSON.
   */
  public function index(Request $request) {
    $path = $this->decoupledKit->checkPath($request);
    if ($path == '/') {
      $path = '<front>';
    }

    // Mode.
    $config = $this->config('decoupled_kit_block.config');
    $mode_from_config = $config->get('mode');
    $mode = $request->query->get('mode') ?? $mode_from_config;
    if (!in_array($mode, ['link', 'data'])) {
      $mode = $mode_from_config;
    }

    // Get from cache.
    $cid = sprintf('decoupled_kit_block:%s:%s', $mode, str_replace('/', '__', $path));
    $cache = $this->cache()->get($cid);
    if ($cache) {
      return new JsonResponse($cache->data);
    }

    $visibled = [];
    $blocks = $this->getBlocks();
    if (!empty($blocks)) {
      foreach ($blocks as $id => $block) {
        $isVisible = $this->blockVisibleForPage($block, $path);
        if ($isVisible) {
          $uuid = $block->uuid();
          $plugin = $block->getPluginId();
          $region = $block->getRegion();
          $weight = $block->getWeight();
          $data = ($mode == 'data') ? $this->getData($plugin)
            : Url::fromRoute('decoupled_kit_block.data', ['plugin' => $plugin])->toString();
          $label = $block->label();

          $visibled[$id] = [
            'region' => $region,
            'label' => $label,
            'link' => "/jsonapi/block/block/$uuid",
            'uuid' => $uuid,
            'plugin' => $plugin,
            'weight' => $weight,
            'data' => $data,
            'settings' => $block->get('settings'),
          ];
        }
      }
    }

    $array = [
      'path' => $path,
      'mode' => $mode,
      'data' => $visibled,
    ];

    // Save to cache.
    $this->cache()->set($cid, $array, CacheBackendInterface::CACHE_PERMANENT, ['block_list', 'block_content_list']);

    return new JsonResponse($array);
  }

  /**
   * Block objects list.
   *
   * @return array
   *   Blocks array.
   */
  protected function getBlocks() {
    $blocksManager = $this->entityTypeManager()->getStorage('block');
    $theme = $this->themeManager->getActiveTheme()->getName();

    $regions = system_region_list($theme, BlockRepositoryInterface::REGIONS_VISIBLE);

    $blocks = [];
    foreach ($regions as $key => $region) {
      $region_blocks = $blocksManager->loadByProperties(
        [
          'theme' => $theme,
          'region' => $key,
        ]
      );

      if (!empty($region_blocks)) {
        $region_blocks = (array) $region_blocks;
        $blocks = array_merge($blocks, $region_blocks);
      }
    }

    return $blocks;
  }

  /**
   * Visibility check.
   *
   * @param object $block
   *   Block object.
   * @param string $input_path
   *   Path to the checking page.
   *
   * @return bool
   *   Is the block visibility?
   */
  protected function blockVisibleForPage($block, $input_path) {
    $visibility = $block->getVisibility();
    if (empty($visibility)) {
      return TRUE;
    }

    $request_path = $visibility['request_path'];
    if (empty($request_path)) {
      return TRUE;
    }

    // Hide for these pages.
    $negate = !empty($request_path['negate']);

    if (empty($request_path['pages'])) {
      // If negate then prohibited for all pages.
      return !$negate;
    }

    foreach (explode("\n", $request_path['pages']) as $alias) {
      $alias = mb_strtolower(trim($alias));

      $match = ($alias == $input_path)
              || $this->pathMatcher->matchPath($alias, $input_path);
      if ($match) {
        return !$negate;
      }
    }

    return FALSE;
  }

  /**
   * JSON content of the block.
   *
   * @param string $plugin
   *   Blocks plugin.
   *
   * @return string
   *   JSON with block data.
   */
  public function getJsonData($plugin) {
    $data = $this->getData($plugin);
    return new JsonResponse(
        [
          'plugin' => $plugin,
          'data' => $data,
        ]
    );
  }

  /**
   * Blocks data.
   *
   * @param string $plugin
   *   Blocks plugin.
   *
   * @return array
   *   Data for block plugin.
   */
  protected function getData($plugin) {
    $data = [];
    list($plugin_id, $plugin_data) = explode(':', $plugin);

    switch ($plugin_id) {
      case 'views_block':
        $data = $this->getViewsBlockData($plugin_data);
        break;

      default:
        break;
    }

    return $data;
  }

  /**
   * View content.
   *
   * @param string $args
   *   View argument.
   *
   * @return array
   *   Views block data.
   */
  protected function getViewsBlockData($args) {
    list($view_name, $view_display) = explode('-', $args);
    if (empty($view_display)) {
      $view_display = 'default';
    }

    $view = views_embed_view($view_name, $view_display);
    if (!$view) {
      return [];
    }

    return $this->renderer->render($view, FALSE);
  }

}
